@@ -18,15 +18,14 @@ import asyncclick as click
 import pyshark
 from cryptography.hazmat.primitives import padding
 
-from kasa.credentials import Credentials
+from kasa.credentials import DEFAULT_CREDENTIALS, Credentials, get_default_credentials
 from kasa.deviceconfig import (
     DeviceConfig,
     DeviceConnectionParameters,
     DeviceEncryptionType,
     DeviceFamily,
 )
-from kasa.klaptransport import KlapEncryptionSession, KlapTransportV2
-from kasa.protocol import DEFAULT_CREDENTIALS, get_default_credentials
+from kasa.transports.klaptransport import KlapEncryptionSession, KlapTransportV2
 
 
 def _get_seq_from_query(packet):
