@@ -6,10 +6,11 @@ import logging
 import time
 from typing import Any
 
+from ..device import DeviceInfo
 from ..device_type import DeviceType
 from ..deviceconfig import DeviceConfig
 from ..protocols.smartprotocol import SmartProtocol, _ChildProtocolWrapper
-from .smartdevice import SmartDevice
+from .smartdevice import ComponentsRaw, SmartDevice
 from .smartmodule import SmartModule
 
 _LOGGER = logging.getLogger(__name__)
@@ -37,7 +38,7 @@ class SmartChildDevice(SmartDevice):
         self,
         parent: SmartDevice,
         info: dict,
-        component_info: dict,
+        component_info_raw: ComponentsRaw,
         *,
         config: DeviceConfig | None = None,
         protocol: SmartProtocol | None = None,
@@ -47,7 +48,24 @@ class SmartChildDevice(SmartDevice):
         super().__init__(parent.host, config=parent.config, protocol=_protocol)
         self._parent = parent
         self._update_internal_state(info)
-        self._components = component_info
+        self._components_raw = component_info_raw
+        self._components = self._parse_components(self._components_raw)
+
+    @property
+    def device_info(self) -> DeviceInfo:
+        """Return device info.
+
+        Child device does not have it info and components in _last_update so
+        this overrides the base implementation to call _get_device_info with
+        info and components combined as they would be in _last_update.
+        """
+        return self._get_device_info(
+            {
+                "get_device_info": self._info,
+                "component_nego": self._components_raw,
+            },
+            None,
+        )
 
     async def update(self, update_children: bool = True) -> None:
         """Update child module info.
@@ -84,7 +102,7 @@ class SmartChildDevice(SmartDevice):
         cls,
         parent: SmartDevice,
         child_info: dict,
-        child_components: dict,
+        child_components_raw: ComponentsRaw,
         protocol: SmartProtocol | None = None,
         *,
         last_update: dict | None = None,
@@ -97,7 +115,7 @@ class SmartChildDevice(SmartDevice):
         derived from the parent.
         """
         child: SmartChildDevice = cls(
-            parent, child_info, child_components, protocol=protocol
+            parent, child_info, child_components_raw, protocol=protocol
         )
         if last_update:
             child._last_update = last_update
