@@ -390,13 +390,12 @@ async def test_device_update_from_new_discovery_info(discovery_mock):
     device_class = Discover._get_device_class(discovery_data)
     device = device_class("127.0.0.1")
     discover_info = DiscoveryResult.from_dict(discovery_data["result"])
-    discover_dump = discover_info.to_dict()
-    model, _, _ = discover_dump["device_model"].partition("(")
-    discover_dump["model"] = model
-    device.update_from_discover_info(discover_dump)
 
-    assert device.mac == discover_dump["mac"].replace("-", ":")
-    assert device.model == model
+    device.update_from_discover_info(discovery_data["result"])
+
+    assert device.mac == discover_info.mac.replace("-", ":")
+    no_region_model, _, _ = discover_info.device_model.partition("(")
+    assert device.model == no_region_model
 
     # TODO implement requires_update for SmartDevice
     if isinstance(device, IotDevice):
